"""
Simple KVDB 高级使用示例
"""

import threading
import time
from simple_kvdb import KVDB, KVDBError

def concurrent_access_example():
    """并发访问示例"""
    print("=== 并发访问示例 ===")
    
    db = KVDB("concurrent_db.json")
    
    def worker(worker_id):
        """工作线程函数"""
        for i in range(10):
            key = f"worker:{worker_id}:{i}"
            value = f"value_from_worker_{worker_id}_{i}"
            db.set(key, value)
            time.sleep(0.01)  # 模拟工作
    
    # 创建多个线程
    threads = []
    for i in range(5):
        thread = threading.Thread(target=worker, args=(i,))
        threads.append(thread)
        thread.start()
    
    # 等待所有线程完成
    for thread in threads:
        thread.join()
    
    print(f"并发写入完成，数据库大小: {db.size()}")
    
    # 验证数据完整性
    expected_keys = 50  # 5个线程 * 10个键
    actual_keys = db.size()
    print(f"预期键数: {expected_keys}, 实际键数: {actual_keys}")
    print("并发访问示例完成\n")

def backup_and_restore_example():
    """备份和恢复示例"""
    print("=== 备份和恢复示例 ===")
    
    db = KVDB("backup_db.json")
    
    # 设置一些数据
    test_data = {
        "important:config": {"theme": "dark", "language": "zh-CN"},
        "user:profile": {"name": "张三", "email": "zhangsan@example.com"},
        "stats:visits": 1500
    }
    
    for key, value in test_data.items():
        db.set(key, value)
    
    print("原始数据设置完成")
    
    # 创建备份
    try:
        backup_path = db.backup()
        print(f"备份创建成功: {backup_path}")
    except KVDBError as e:
        print(f"备份失败: {e}")
    
    # 模拟数据损坏
    print("模拟数据损坏...")
    db.clear()
    db.set("corrupted", "data")
    
    print("数据已清空并设置损坏数据")
    
    # 从备份恢复（这里只是演示，实际需要手动操作）
    print("备份文件可用于手动恢复数据")
    print("备份和恢复示例完成\n")

def performance_example():
    """性能测试示例"""
    print("=== 性能测试示例 ===")
    
    # 测试自动保存模式
    print("测试自动保存模式:")
    start_time = time.time()
    
    db_auto = KVDB("performance_auto.json", auto_save=True)
    for i in range(1000):
        db_auto.set(f"key_{i}", f"value_{i}")
    
    auto_time = time.time() - start_time
    print(f"自动保存模式耗时: {auto_time:.3f}秒")
    
    # 测试手动保存模式
    print("\n测试手动保存模式:")
    start_time = time.time()
    
    db_manual = KVDB("performance_manual.json", auto_save=False)
    for i in range(1000):
        db_manual.set(f"key_{i}", f"value_{i}")
    
    # 手动保存
    db_manual.save()
    manual_time = time.time() - start_time
    print(f"手动保存模式耗时: {manual_time:.3f}秒")
    
    print(f"性能提升: {((auto_time - manual_time) / auto_time * 100):.1f}%")
    print("性能测试示例完成\n")

def complex_data_example():
    """复杂数据结构示例"""
    print("=== 复杂数据结构示例 ===")
    
    db = KVDB("complex_db.json")
    
    # 存储复杂的嵌套数据结构
    complex_data = {
        "application": {
            "name": "我的应用",
            "version": "1.0.0",
            "settings": {
                "theme": "dark",
                "language": "zh-CN",
                "notifications": True
            }
        },
        "users": [
            {
                "id": 1,
                "name": "张三",
                "email": "zhangsan@example.com",
                "preferences": {
                    "theme": "light",
                    "notifications": False
                }
            },
            {
                "id": 2,
                "name": "李四",
                "email": "lisi@example.com",
                "preferences": {
                    "theme": "dark",
                    "notifications": True
                }
            }
        ],
        "statistics": {
            "total_users": 2,
            "active_users": 1,
            "daily_visits": [100, 150, 200, 180, 220]
        }
    }
    
    # 存储复杂数据
    db.set("app_data", complex_data)
    
    # 读取和修改复杂数据
    app_data = db.get("app_data")
    print(f"应用名称: {app_data['application']['name']}")
    print(f"用户数量: {app_data['statistics']['total_users']}")
    
    # 原子更新复杂数据
    def update_user_count(data):
        if 'statistics' in data:
            data['statistics']['total_users'] += 1
        return data
    
    updated_data = db.atomic_update("app_data", update_user_count)
    print(f"更新后用户数量: {updated_data['statistics']['total_users']}")
    
    print("复杂数据结构示例完成\n")

def error_handling_example():
    """错误处理示例"""
    print("=== 错误处理示例 ===")
    
    try:
        # 尝试创建无效的数据库文件
        db = KVDB("/invalid/path/db.json")
        print("数据库创建成功")
    except KVDBError as e:
        print(f"数据库创建失败: {e}")
    
    # 测试不存在的键
    db = KVDB("error_test.json")
    value = db.get("nonexistent_key", "默认值")
    print(f"获取不存在的键: {value}")
    
    # 测试删除不存在的键
    result = db.delete("nonexistent_key")
    print(f"删除不存在的键: {result}")
    
    print("错误处理示例完成\n")

def main():
    """运行所有高级示例"""
    print("Simple KVDB 高级使用示例\n")
    
    # 运行各种示例
    concurrent_access_example()
    backup_and_restore_example()
    performance_example()
    complex_data_example()
    error_handling_example()
    
    print("所有高级示例完成！")

if __name__ == "__main__":
    main() 