"""
KVDB - 键值存储数据库的核心实现
"""

import json
import os
import shutil
import threading
import time
from pathlib import Path
from typing import Any, Dict, List, Optional, Union, Callable


class KVDBError(Exception):
    """KVDB 数据库异常"""
    pass


class KVDB:
    """
    轻量级键值存储数据库
    
    使用JSON文件存储数据，提供简单易用的API接口。
    支持线程安全操作、自动备份、批量操作等功能。
    """
    
    def __init__(self, file_path: str, auto_save: bool = True, backup_count: int = 3):
        """
        初始化数据库
        
        Args:
            file_path: 数据库文件路径
            auto_save: 是否自动保存，默认为True
            backup_count: 备份文件数量，默认为3
        """
        self.file_path = Path(file_path)
        self.auto_save = auto_save
        self.backup_count = backup_count
        self._data: Dict[str, Any] = {}
        self._lock = threading.RLock()
        self._modified = False
        self._last_save = 0
        
        # 确保目录存在
        self.file_path.parent.mkdir(parents=True, exist_ok=True)
        
        # 加载现有数据
        self.load()
    
    def set(self, key: str, value: Any) -> None:
        """
        设置键值对
        
        Args:
            key: 键名
            value: 值
        """
        with self._lock:
            self._data[key] = value
            self._modified = True
            if self.auto_save:
                self._auto_save()
    
    def get(self, key: str, default: Any = None) -> Any:
        """
        获取值
        
        Args:
            key: 键名
            default: 默认值，当键不存在时返回
            
        Returns:
            对应的值或默认值
        """
        with self._lock:
            return self._data.get(key, default)
    
    def delete(self, key: str) -> bool:
        """
        删除键值对
        
        Args:
            key: 键名
            
        Returns:
            是否成功删除
        """
        with self._lock:
            if key in self._data:
                del self._data[key]
                self._modified = True
                if self.auto_save:
                    self._auto_save()
                return True
            return False
    
    def exists(self, key: str) -> bool:
        """
        检查键是否存在
        
        Args:
            key: 键名
            
        Returns:
            键是否存在
        """
        with self._lock:
            return key in self._data
    
    def keys(self) -> List[str]:
        """
        获取所有键
        
        Returns:
            所有键的列表
        """
        with self._lock:
            return list(self._data.keys())
    
    def values(self) -> List[Any]:
        """
        获取所有值
        
        Returns:
            所有值的列表
        """
        with self._lock:
            return list(self._data.values())
    
    def items(self) -> List[tuple]:
        """
        获取所有键值对
        
        Returns:
            所有键值对的列表
        """
        with self._lock:
            return list(self._data.items())
    
    def size(self) -> int:
        """
        获取数据库大小
        
        Returns:
            键值对的数量
        """
        with self._lock:
            return len(self._data)
    
    def clear(self) -> None:
        """
        清空数据库
        """
        with self._lock:
            self._data.clear()
            self._modified = True
            if self.auto_save:
                self._auto_save()
    
    def save(self) -> None:
        """
        手动保存数据到文件
        """
        with self._lock:
            if not self._modified:
                return
            
            try:
                # 创建临时文件
                temp_file = self.file_path.with_suffix('.tmp')
                
                # 写入数据到临时文件
                with open(temp_file, 'w', encoding='utf-8') as f:
                    json.dump(self._data, f, ensure_ascii=False, indent=2)
                
                # 原子性地替换文件
                temp_file.replace(self.file_path)
                
                self._modified = False
                self._last_save = time.time()
                
            except Exception as e:
                raise KVDBError(f"保存数据失败: {e}")
    
    def load(self) -> None:
        """
        从文件加载数据
        """
        with self._lock:
            if not self.file_path.exists():
                self._data = {}
                return
            
            try:
                with open(self.file_path, 'r', encoding='utf-8') as f:
                    self._data = json.load(f)
            except json.JSONDecodeError as e:
                raise KVDBError(f"JSON格式错误: {e}")
            except Exception as e:
                raise KVDBError(f"加载数据失败: {e}")
    
    def backup(self) -> str:
        """
        创建数据库备份
        
        Returns:
            备份文件路径
        """
        with self._lock:
            if not self.file_path.exists():
                raise KVDBError("数据库文件不存在，无法备份")
            
            timestamp = int(time.time())
            backup_path = self.file_path.with_suffix(f'.backup.{timestamp}.json')
            
            try:
                shutil.copy2(self.file_path, backup_path)
                
                # 清理旧的备份文件
                self._cleanup_backups()
                
                return str(backup_path)
            except Exception as e:
                raise KVDBError(f"创建备份失败: {e}")
    
    def set_many(self, data: Dict[str, Any]) -> None:
        """
        批量设置键值对
        
        Args:
            data: 要设置的键值对字典
        """
        with self._lock:
            self._data.update(data)
            self._modified = True
            if self.auto_save:
                self._auto_save()
    
    def get_many(self, keys: List[str]) -> Dict[str, Any]:
        """
        批量获取值
        
        Args:
            keys: 键名列表
            
        Returns:
            键值对字典
        """
        with self._lock:
            return {key: self._data.get(key) for key in keys}
    
    def atomic_update(self, key: str, func: Callable[[Any], Any]) -> Any:
        """
        原子更新操作
        
        Args:
            key: 键名
            func: 更新函数，接收当前值并返回新值
            
        Returns:
            更新后的值
        """
        with self._lock:
            current_value = self._data.get(key)
            new_value = func(current_value)
            self._data[key] = new_value
            self._modified = True
            if self.auto_save:
                self._auto_save()
            return new_value
    
    def export(self) -> Dict[str, Any]:
        """
        导出所有数据
        
        Returns:
            所有数据的字典副本
        """
        with self._lock:
            return self._data.copy()
    
    def import_data(self, data: Dict[str, Any]) -> None:
        """
        导入数据
        
        Args:
            data: 要导入的数据字典
        """
        with self._lock:
            self._data.update(data)
            self._modified = True
            if self.auto_save:
                self._auto_save()
    
    def _auto_save(self) -> None:
        """
        自动保存（如果启用）
        """
        if self.auto_save and self._modified:
            # 避免过于频繁的保存
            if time.time() - self._last_save > 0.1:  # 100ms 间隔
                self.save()
    
    def _cleanup_backups(self) -> None:
        """
        清理旧的备份文件
        """
        if self.backup_count <= 0:
            return
        
        backup_files = []
        for file in self.file_path.parent.glob(f"{self.file_path.stem}.backup.*.json"):
            backup_files.append((file, file.stat().st_mtime))
        
        # 按修改时间排序，保留最新的几个
        backup_files.sort(key=lambda x: x[1], reverse=True)
        
        # 删除多余的备份文件
        for file, _ in backup_files[self.backup_count:]:
            try:
                file.unlink()
            except Exception:
                pass  # 忽略删除失败的错误
    
    def __enter__(self):
        """上下文管理器入口"""
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口，确保数据保存"""
        if self._modified:
            self.save()
    
    def __len__(self) -> int:
        """返回数据库大小"""
        return self.size()
    
    def __contains__(self, key: str) -> bool:
        """检查键是否存在"""
        return self.exists(key)
    
    def __getitem__(self, key: str) -> Any:
        """通过索引获取值"""
        value = self.get(key)
        if value is None and key not in self._data:
            raise KeyError(key)
        return value
    
    def __setitem__(self, key: str, value: Any) -> None:
        """通过索引设置值"""
        self.set(key, value)
    
    def __delitem__(self, key: str) -> None:
        """通过索引删除值"""
        if not self.delete(key):
            raise KeyError(key) 