"""
KVDB 测试文件
"""

import json
import os
import tempfile
import time
import unittest
import shutil
from pathlib import Path

from simple_kvdb import KVDB, KVDBError


class TestKVDB(unittest.TestCase):
    """KVDB 测试类"""
    
    def setUp(self):
        """测试前准备"""
        self.temp_dir = tempfile.mkdtemp()
        self.db_path = os.path.join(self.temp_dir, "test_db.json")
        self.db = KVDB(self.db_path, auto_save=True)
    
    def tearDown(self):
        """测试后清理"""
        # 删除测试文件
        if os.path.exists(self.db_path):
            try:
                os.remove(self.db_path)
            except:
                pass
        
        # 删除临时目录及其所有内容
        if os.path.exists(self.temp_dir):
            try:
                shutil.rmtree(self.temp_dir)
            except:
                pass
    
    def test_basic_operations(self):
        """测试基本操作"""
        # 测试设置和获取
        self.db.set("key1", "value1")
        self.assertEqual(self.db.get("key1"), "value1")
        
        # 测试默认值
        self.assertEqual(self.db.get("nonexistent", "default"), "default")
        
        # 测试删除
        self.assertTrue(self.db.delete("key1"))
        self.assertFalse(self.db.exists("key1"))
        
        # 测试删除不存在的键
        self.assertFalse(self.db.delete("nonexistent"))
    
    def test_complex_data_types(self):
        """测试复杂数据类型"""
        # 测试字典
        user_data = {"name": "张三", "age": 25, "city": "北京"}
        self.db.set("user:1", user_data)
        retrieved = self.db.get("user:1")
        self.assertEqual(retrieved, user_data)
        
        # 测试列表
        numbers = [1, 2, 3, 4, 5]
        self.db.set("numbers", numbers)
        self.assertEqual(self.db.get("numbers"), numbers)
        
        # 测试嵌套结构
        complex_data = {
            "users": [
                {"id": 1, "name": "张三"},
                {"id": 2, "name": "李四"}
            ],
            "settings": {
                "theme": "dark",
                "language": "zh-CN"
            }
        }
        self.db.set("complex", complex_data)
        self.assertEqual(self.db.get("complex"), complex_data)
    
    def test_bulk_operations(self):
        """测试批量操作"""
        # 批量设置
        data = {
            "key1": "value1",
            "key2": "value2",
            "key3": "value3"
        }
        self.db.set_many(data)
        
        # 验证所有键都存在
        for key, value in data.items():
            self.assertEqual(self.db.get(key), value)
        
        # 批量获取
        keys = ["key1", "key2", "key3"]
        retrieved = self.db.get_many(keys)
        expected = {key: data[key] for key in keys}
        self.assertEqual(retrieved, expected)
    
    def test_atomic_operations(self):
        """测试原子操作"""
        # 设置初始值
        self.db.set("counter", 0)
        
        # 原子递增
        self.db.atomic_update("counter", lambda x: (x or 0) + 1)
        self.assertEqual(self.db.get("counter"), 1)
        
        # 再次递增
        self.db.atomic_update("counter", lambda x: x + 1)
        self.assertEqual(self.db.get("counter"), 2)
        
        # 测试不存在的键
        self.db.atomic_update("new_counter", lambda x: (x or 0) + 10)
        self.assertEqual(self.db.get("new_counter"), 10)
    
    def test_persistence(self):
        """测试数据持久化"""
        # 设置数据
        test_data = {"test": "data", "number": 42}
        for key, value in test_data.items():
            self.db.set(key, value)
        
        # 创建新的数据库实例，应该加载相同的数据
        new_db = KVDB(self.db_path)
        for key, value in test_data.items():
            self.assertEqual(new_db.get(key), value)
    
    def test_backup_functionality(self):
        """测试备份功能"""
        # 设置一些数据
        self.db.set("backup_test", "backup_value")
        self.db.save()  # 确保数据已保存
        
        # 创建备份
        backup_path = self.db.backup()
        
        # 验证备份文件存在
        self.assertTrue(os.path.exists(backup_path))
        
        # 验证备份文件内容正确
        with open(backup_path, 'r', encoding='utf-8') as f:
            backup_data = json.load(f)
        self.assertEqual(backup_data, {"backup_test": "backup_value"})
        
        # 清理备份文件
        try:
            os.remove(backup_path)
        except:
            pass
    
    def test_context_manager(self):
        """测试上下文管理器"""
        with KVDB(self.db_path) as db:
            db.set("context_test", "context_value")
            # 退出时应该自动保存
        
        # 验证数据已保存
        new_db = KVDB(self.db_path)
        self.assertEqual(new_db.get("context_test"), "context_value")
    
    def test_dictionary_interface(self):
        """测试字典接口"""
        # 测试 __getitem__ 和 __setitem__
        self.db["dict_key"] = "dict_value"
        self.assertEqual(self.db["dict_key"], "dict_value")
        
        # 测试 __contains__
        self.assertTrue("dict_key" in self.db)
        self.assertFalse("nonexistent" in self.db)
        
        # 测试 __len__
        self.assertEqual(len(self.db), 1)
        
        # 测试 __delitem__
        del self.db["dict_key"]
        self.assertFalse("dict_key" in self.db)
        
        # 测试 KeyError
        with self.assertRaises(KeyError):
            _ = self.db["nonexistent"]
    
    def test_error_handling(self):
        """测试错误处理"""
        # 测试无效的JSON文件
        invalid_json_path = os.path.join(self.temp_dir, "invalid.json")
        with open(invalid_json_path, 'w') as f:
            f.write("invalid json content")
        
        with self.assertRaises(KVDBError):
            KVDB(invalid_json_path)
    
    def test_auto_save_disabled(self):
        """测试禁用自动保存"""
        db = KVDB(self.db_path, auto_save=False)
        
        # 设置数据但不自动保存
        db.set("test_key", "test_value")
        
        # 创建新的数据库实例，应该看不到数据
        new_db = KVDB(self.db_path)
        self.assertIsNone(new_db.get("test_key"))
        
        # 手动保存后应该能看到数据
        db.save()
        new_db.load()
        self.assertEqual(new_db.get("test_key"), "test_value")
    
    def test_clear_operation(self):
        """测试清空操作"""
        # 设置一些数据
        self.db.set("key1", "value1")
        self.db.set("key2", "value2")
        self.assertEqual(self.db.size(), 2)
        
        # 清空数据库
        self.db.clear()
        self.assertEqual(self.db.size(), 0)
        self.assertFalse(self.db.exists("key1"))
        self.assertFalse(self.db.exists("key2"))
    
    def test_export_import(self):
        """测试导出导入功能"""
        # 设置测试数据
        test_data = {
            "user1": {"name": "张三", "age": 25},
            "user2": {"name": "李四", "age": 30},
            "settings": {"theme": "dark", "language": "zh-CN"}
        }
        
        for key, value in test_data.items():
            self.db.set(key, value)
        
        # 导出数据
        exported = self.db.export()
        self.assertEqual(exported, test_data)
        
        # 清空数据库
        self.db.clear()
        self.assertEqual(self.db.size(), 0)
        
        # 导入数据
        self.db.import_data(test_data)
        self.assertEqual(self.db.size(), len(test_data))
        
        # 验证数据正确
        for key, value in test_data.items():
            self.assertEqual(self.db.get(key), value)


if __name__ == "__main__":
    unittest.main() 